import subprocess
import threading

from gi.repository import GLib

class ApplicationManager:
    @staticmethod
    def get_available_applications(callback):
        threading.Thread(target=ApplicationManager._get_available_applications, daemon=True, args=(callback,)).start()

    @staticmethod
    def _get_available_applications(callback):
        process = subprocess.Popen(["epm", "play", "--list-all"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

        def read_output():
            stdout, stderr = process.communicate()
            if process.returncode == 0:
                applications = ApplicationManager.parse_applications_output(stdout)
                GLib.idle_add(callback, applications)
            else:
                GLib.idle_add(callback, [], stderr)  # Pass error message

        threading.Thread(target=read_output, daemon=True).start()

    @staticmethod
    def get_installed_applications(callback):
        threading.Thread(target=ApplicationManager._get_installed_applications, daemon=True, args=(callback,)).start()

    @staticmethod
    def _get_installed_applications(callback):
        process = subprocess.Popen(["epm", "play", "--list"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

        def read_output():
            stdout, stderr = process.communicate()
            if process.returncode == 0:
                installed_apps = ApplicationManager.parse_installed_applications_output(stdout)
                GLib.idle_add(callback, installed_apps)
            else:
                GLib.idle_add(callback, [], stderr)  # Pass error message

        threading.Thread(target=read_output, daemon=True).start()

    @staticmethod
    def parse_applications_output(output):
        applications = []

        for line in output.splitlines()[1:]:  # Пропустить заголовок
            parts = line.split(' - ', 1)  # Ограничить сплит до 2 частей
            if len(parts) == 2:
                name, dscr = map(str.strip, parts)
                applications.append({
                    'name': name.replace('&', '&amp;'),
                    'dscr': dscr.replace('&', '&amp;')
                })
            else:
                # Логгирование некорректной строки для диагностики
                print(f"Неправильный формат строки: {line}")

        return applications

    @staticmethod
    def parse_installed_applications_output(output):
        lines = output.splitlines()[1:]
        return [line.split(' - ')[0].strip().split()[0] for line in lines]
